@@ -1,13 +1,11 @@
 package HPC::Runner::Command;
 
-use MooseX::App qw(Color Config);
+use MooseX::App qw(Color);
 
 with 'HPC::Runner::Command::Utils::Plugin';
+with 'HPC::Runner::Command::Utils::ManyConfigs';
 
-#This is not working and I'm not sure why...
-#use HPC::Runner::Command qw(ConfigHome);
-
-our $VERSION = '3.0.1';
+our $VERSION = '3.1.0';
 
 app_strict 0;
 
@@ -45,8 +43,6 @@ logging results.
 
 Please see the complete documentation at L<HPC::Runner::Command GitBooks | https://jerowe.gitbooks.io/hpc-runner-command-docs/content/>
 
-This is a beta release of the HPC::Runner::Command libraries. The codebase is stable, but we will be adding more error handling, bug fixes, etc before the official 3.1.0 release.
-
 =head1 Quick Start - Create a New Project
 
 You can create a new project, with a sane directory structure by using
@@ -194,14 +190,14 @@ This software was previously released under L<HPC::Runner>. L<HPC::Runner::Comma
 
 =head1 Acknowledgements
 
-Before Version 2.41
-
-This module was originally developed at and for Weill Cornell Medical College in Qatar within ITS Advanced Computing Team. With approval from WCMC-Q, this information was generalized and put on github, for which the authors would like to express their gratitude.
-
 As of Version 2.41:
 
 This modules continuing development is supported by NYU Abu Dhabi in the Center for Genomics and Systems Biology. With approval from NYUAD, this information was generalized and put on bitbucket, for which the authors would like to express their gratitude.
 
+Before Version 2.41
+
+This module was originally developed at and for Weill Cornell Medical College in Qatar within ITS Advanced Computing Team. With approval from WCMC-Q, this information was generalized and put on github, for which the authors would like to express their gratitude.
+
 =head1 COPYRIGHT
 
 Copyright 2016- Jillian Rowe
